// Copyright (C) 2016 Parrot SA
//
//    Redistribution and use in source and binary forms, with or without
//    modification, are permitted provided that the following conditions
//    are met:
//    * Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in
//      the documentation and/or other materials provided with the
//      distribution.
//    * Neither the name of Parrot nor the names
//      of its contributors may be used to endorse or promote products
//      derived from this software without specific prior written
//      permission.
//
//    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
//    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
//    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
//    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
//    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
//    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//    SUCH DAMAGE.
//
//  Created by Nicolas CHRISTE, <nicolas.christe@parrot.com>

import Foundation

/**
 A Drone Controller that runs in the live view to land the drone
 */
public class DroneLandController {

    /** Ble instance */
    fileprivate var ble: DroneBle?
    /** Protocol instance */
    fileprivate var droneProtocol: DroneProtocol?


    /**
     Constructor
     */
    public init() {
    }

    deinit {

    }

    /**
     Send land command
     */
    public func landNow(droneName: String) {
        ble = DroneBle(droneId: DroneId(name: droneName), queue: nil, delegate: self)
    }
}

/** Add ble delegate facet */
extension DroneLandController: DroneBleDelegate {
    func droneBleDidConnect() {
        droneProtocol = DroneProtocol(droneBle: ble!, delegate: self)
        // don't run a full connection sequence, just send land command 3 times
        droneProtocol?.land()
        droneProtocol?.land()
        droneProtocol?.land()
        droneProtocol = nil
        ble = nil
    }

    func droneBleDidDisconnect() {
        droneProtocol = nil
        ble = nil
    }
}


/** Add protocol delegate facet */
extension DroneLandController: DroneProtocolDelegate {

    func protocolDidConnect() {
    }

    func protocolDidDisconnect() {
    }

    func flyingStateDidChange(state: FlyingState) {
    }

    func lowBattery() {
    }

    func pictureStateDidChange(ready: Bool) {
    }

    func headlightAvailable() {
    }

    func accessoryLight(available: Bool, id: UInt8, state: Lights.State) {
    }

    func accessoryClaw(available: Bool, id: UInt8, opened: Bool ) {
    }

    func accessoryGun(available: Bool, id: UInt8, ready: Bool) {
    }
}

